package node

import (
	"encoding/json"
	"io/ioutil"
	"os"
	"path/filepath"
	"strings"
)

const (
	persistFile = ".persist.json"
)

type PersistConfig struct {
	ChequeBookTx    string `json:"cheque_book_tx"`
	TxNextBlockHash string `json:"tx_next_block_hash"`
	Datadir         string
}

func (c *Bee) initPersistConfig(datadir string) (err error) {
	cfgfile := filepath.Join(datadir, persistFile)

	var cfg PersistConfig
	content, err := ioutil.ReadFile(cfgfile)
	if err == nil {
		json.Unmarshal(content, &cfg)
	}
	cfg.Datadir = datadir
	c.persistCfg = &cfg

	return nil
}

func (c *Bee) SavePersistConfig() (err error) {
	basepath := c.persistCfg.Datadir
	cfgfile := filepath.Join(basepath, persistFile)

	bytes, _ := json.Marshal(c.persistCfg)
	err = ioutil.WriteFile(cfgfile+".tmp", bytes, 0600)
	if err != nil {
		return err
	}
	err = os.Rename(cfgfile+".tmp", cfgfile)
	if err != nil {
		os.Remove(cfgfile + ".tmp")
		return err
	}
	return nil
}

func (c *Bee) GetNextBlock(txhash string) string {
	if c.persistCfg.TxNextBlockHash != "" && strings.Compare(txhash, c.persistCfg.ChequeBookTx) == 0 {
		return c.persistCfg.TxNextBlockHash
	}
	return ""
}

func (c *Bee) UpdateTxNextBlock(txhash string, nextBlock string) (err error) {
	c.persistCfg.ChequeBookTx = txhash
	c.persistCfg.TxNextBlockHash = nextBlock
	return c.SavePersistConfig()
}
